@@ -22,6 +22,7 @@ Student Intervention browser views.
 import csv
 from StringIO import StringIO
 import tempfile
+import urllib
 import zipfile
 
 from zope.app.form.browser.editview import EditView
@@ -66,7 +67,6 @@ from schooltool.securitypolicy.crowds import AdministrationCrowd
 from schooltool.skin.skin import NavigationViewlet
 from schooltool.skin import flourish
 from schooltool.skin.flourish.viewlet import ViewletManager
-import schooltool.skin.flourish.page
 from schooltool.table.catalog import IndexedTableFormatter, FilterWidget
 from schooltool.table.catalog import IndexedGetterColumn
 from schooltool.table.catalog import IndexedLocaleAwareGetterColumn
@@ -1080,7 +1080,7 @@ class IMessageAddForm(interfaces.IInterventionMessage):
 
     email = Bool(
         title=_(u"Email"),
-        description=_(u"You can automatically send an email of this message."),
+        description=_(u"Send an email to the people selected above."),
         required=False,
         default=True)
 
@@ -1092,7 +1092,7 @@ class FlourishInterventionMessageAddView(flourish.form.AddForm):
     legend = _('Message Details')
 
     fields = field.Fields(IMessageAddForm)
-    fields = fields.select('recipients', 'body', 'email')
+    fields = fields.select('recipients', 'subject', 'body', 'email')
 
     @property
     def title(self):
@@ -1118,7 +1118,8 @@ class FlourishInterventionMessageAddView(flourish.form.AddForm):
             if data.get('email'):
                 email = sendmail.sendInterventionMessageEmail(obj)
                 if email.status_code is None:
-                    info = ', '.join(email.to_addresses)
+                    info = ', '.join([urllib.quote(to)
+                                      for to in email.to_addresses])
                 else:
                     info = email.__name__
                 url = '%s?email=%s' % (self.nextURL(), info)
@@ -1138,7 +1139,8 @@ class FlourishInterventionMessageAddView(flourish.form.AddForm):
     def create(self, data):
         sender = IContact(self.request.principal._person)
         message = intervention.InterventionMessage(sender, data['recipients'],
-                                                   data['body'])
+                                                   data['body'],
+                                                   data['subject'])
         return message
 
     def add(self, message):
@@ -1249,6 +1251,7 @@ class InterventionMessagesView(object):
                 'creation_date': added,
                 'heading': heading,
                 'recipients': recipients,
+                'subject': v.subject,
                 'body': v.body,
             }
             self.messages.append(message)
@@ -1288,6 +1291,7 @@ class FlourishInterventionMessagesView(flourish.page.Page):
                 'added': v.created.strftime('%x'),
                 'recipients': ', '.join(intervention.contactsName(
                                         v.recipients)),
+                'subject': v.subject,
                 'body': v.body,
             })
         return sorted(messages, key=lambda m: m['creation_date'], reverse=True)
@@ -1312,6 +1316,23 @@ class InterventionStatusMessagesView(InterventionMessagesView):
                  mapping={'student': name})
 
 
+class MessagePersonLinks(flourish.page.RefineLinksViewlet):
+    """Message Person links viewlet."""
+
+    @property
+    def title(self):
+        student = removeSecurityProxy(IPerson(self.context))
+        return "%s %s" % (student.first_name, student.last_name)
+
+
+class MessagePersonInterventionsLink(flourish.page.LinkViewlet):
+
+    @property
+    def url(self):
+        student = removeSecurityProxy(IPerson(self.context))
+        return absoluteURL(student, self.request) + '/intervention'
+
+
 class InterventionGoalAddView(app.BaseAddView):
     """View for adding a goal."""
 
@@ -1342,7 +1363,7 @@ class IGoalAddForm(interfaces.IInterventionGoal):
 
     email = Bool(
         title=_(u"Email"),
-        description=_(u"You can automatically send an email of this goal."),
+        description=_(u"Send an email to the people selected above."),
         required=False,
         default=True)
 
@@ -1396,7 +1417,8 @@ class FlourishInterventionGoalAddView(flourish.form.AddForm):
                 emails = sendmail.sendInterventionGoalAddEmail(obj)
                 for email in emails:
                     if email.status_code is None:
-                        info.append(', '.join(email.to_addresses))
+                        info.append(', '.join([urllib.quote(to)
+                                               for to in email.to_addresses]))
                     else:
                         info.append(email.__name__)
                 url = '%s?email=%s' % (self.nextURL(), '|'.join(info))
@@ -1416,9 +1438,10 @@ class FlourishInterventionGoalAddView(flourish.form.AddForm):
 
     def create(self, data):
         creator = IContact(self.request.principal._person)
-        goal = intervention.InterventionGoal(data['presenting_concerns'],
-            data['goal'], data['strengths'], data['indicators'],
-            data['intervention'], data['timeline'], data['persons_responsible'],
+        goal = intervention.InterventionGoal(
+            data['goal'], data['timeline'], data['persons_responsible'],
+            data['presenting_concerns'], data['strengths'], data['indicators'],
+            data['intervention'],
             goal_met=data['goal_met'], follow_up_notes=data['follow_up_notes'],
             creator=creator)
         return goal
@@ -1629,6 +1652,23 @@ class FlourishInterventionGoalsView(flourish.page.Page):
         return absoluteURL(self.context.__parent__, self.request)
 
 
+class GoalPersonLinks(flourish.page.RefineLinksViewlet):
+    """Goal Person links viewlet."""
+
+    @property
+    def title(self):
+        student = removeSecurityProxy(IPerson(self.context))
+        return "%s %s" % (student.first_name, student.last_name)
+
+
+class GoalPersonInterventionsLink(flourish.page.LinkViewlet):
+
+    @property
+    def url(self):
+        student = removeSecurityProxy(IPerson(self.context))
+        return absoluteURL(student, self.request) + '/intervention'
+
+
 class NotifyGoalsView(object):
     """View for notifying persons responsible for goals that
        have come due"""
