@@ -25,12 +25,13 @@ from zope.formlib.interfaces import IBrowserWidget, IInputWidget
 from zope.formlib.interfaces import InputErrors
 from zope.app.form.browser.submit import Update
 from zope.browserpage import ViewPageTemplateFile
-from zope.component import adapter, adapts, getUtility, queryUtility
-from zope.interface import implementer, implements
+from zope.component import adapts, getUtility, queryUtility
+from zope.i18n import translate
+from zope.interface import implements
 from zope.security.proxy import removeSecurityProxy
 
 from z3c.form.converter import BaseDataConverter
-from z3c.form.interfaces import IWidget, IFieldWidget, IDataConverter
+from z3c.form.interfaces import IFieldWidget, IDataConverter
 from z3c.form.widget import Widget, FieldWidget
 
 from schooltool.contact.interfaces import IContactable, IContact, IUniqueFormKey
@@ -47,7 +48,9 @@ from schooltool.intervention import interfaces, intervention
 
 
 class WidgetBase(object):
-    """Base Class for all Widget Classes"""
+    """Base Class for all Widget Classes.
+
+    XXX: why copy from zope.formlib.widget?"""
 
     implements(IBrowserWidget, IInputWidget)
 
@@ -152,7 +155,8 @@ class PersonListWidget(WidgetBase):
                     if IContact(teacher) not in responsible:
                         responsible.append(IContact(teacher))
         notified = [IContact(self.student)]
-        notified += [contact for contact in IContactable(self.student).contacts]
+        notified += [contact for contact in IContactable(self.student).contacts
+                     if contact not in responsible]
         if self.isAddView():
             changed = []
         else:
@@ -194,12 +198,12 @@ class PersonListWidget(WidgetBase):
                 'hasEmail': bool(removeSecurityProxy(choice).email),
                 }
 
-            if choice in notified:
-                self.notified.append(person)
-            elif choice in changed:
-                self.changed.append(person)
-            else:
+            if choice in responsible:
                 self.responsible.append(person)
+            elif choice in notified:
+                self.notified.append(person)
+            else:
+                self.changed.append(person)
 
     def canEmail(self, person):
         return self.enabled and person['hasEmail']
@@ -248,6 +252,9 @@ class PersonListZ3CFormWidget(Widget, PersonListWidget):
     def update(self):
         """See z3c.form.interfaces.IWidget."""
         self.student = IPerson(self.context)
+        sender = IPerson(self.request.principal, None)
+        if sender is not None:
+            sender = IContact(sender)
 
         # establish email capability and error message
         utility = getUtility(IEmailUtility)
@@ -256,8 +263,7 @@ class PersonListZ3CFormWidget(Widget, PersonListWidget):
         if not self.enabled:
             self.message = _('Email server is not enabled')
         else:
-            sender = IContact(self.request.principal._person)
-            if not sender.email:
+            if sender is None or not sender.email:
                 self.message = _('User has no email address')
                 self.enabled = False
 
@@ -267,16 +273,20 @@ class PersonListZ3CFormWidget(Widget, PersonListWidget):
                     for advisor in self.student.advisors]
         groups = IGroupContainer(ISchoolYear(term))
         administrators = [IContact(member)
-                          for member in groups['administrators'].members]
-        responsible = advisors + [contact for contact in administrators
-                                  if contact not in advisors]
+                          for member in groups['administrators'].members
+                          if IContact(member) not in advisors]
+        responsible = advisors + administrators
+        teachers = []
         for section in ISectionContainer(term).values():
             if self.student in section.members:
                 for teacher in section.instructors:
-                    if IContact(teacher) not in responsible:
-                        responsible.append(IContact(teacher))
+                    teacher = IContact(teacher)
+                    if teacher not in responsible:
+                        responsible.append(teacher)
+                        teachers.append(teacher)
         notified = [IContact(self.student)]
-        notified += [contact for contact in IContactable(self.student).contacts]
+        notified += [contact for contact in IContactable(self.student).contacts
+                     if contact not in responsible]
         if self.isAddView():
             changed = []
         else:
@@ -297,20 +307,30 @@ class PersonListZ3CFormWidget(Widget, PersonListWidget):
         self.notified = []
         self.changed = []
         for choice in self.choices:
+            title = ''
             checked = ''
             if choice in valueList:
                 checked = 'checked'
 
-            value = intervention.contactName(choice)
             if choice in advisors:
-                value = '%s (%s)' % (value, _('advisor'))
+                title = _('advisor')
                 if self.isAddView() and not 'SUBMIT' in self.request:
                     checked = 'checked'
-
-            if choice in IContactable(self.student).contacts:
+            elif choice in administrators:
+                title = _('administrator')
+            elif choice in teachers:
+                title = _('teacher')
+            elif choice is IContact(self.student):
+                title = _('student')
+            elif choice in IContactable(self.student).contacts:
                 title = get_relationship_title(self.student, choice)
-                if title:
-                    value = '%s (%s)' % (value, title.lower())
+            elif choice is sender:
+                title = _('self')
+            if title:
+                value = '%s (%s)' % (intervention.contactName(choice),
+                                     translate(title, context=self.request).lower())
+            else:
+                value = intervention.contactName(choice)
 
             person = {
                 'name': IUniqueFormKey(choice),
@@ -319,12 +339,12 @@ class PersonListZ3CFormWidget(Widget, PersonListWidget):
                 'hasEmail': bool(removeSecurityProxy(choice).email),
                 }
 
-            if choice in notified:
-                self.notified.append(person)
-            elif choice in changed:
-                self.changed.append(person)
-            else:
+            if choice in responsible:
                 self.responsible.append(person)
+            elif choice in notified:
+                self.notified.append(person)
+            else:
+                self.changed.append(person)
 
         # finally call the z3c base class
         super(PersonListZ3CFormWidget, self).update()
