@@ -19,7 +19,7 @@
 
 from zope.container.interfaces import IContainer, IContained
 from zope.container.constraints import contains, containers
-from zope.interface import Interface, implements, Attribute
+from zope.interface import Interface, implements
 from zope.schema.interfaces import IList, IBool
 from zope.schema import TextLine, Text, Bool, List, Date, Datetime
 
@@ -109,6 +109,11 @@ class IInterventionMessage(IContained):
         title=_("Recipients"),
         )
 
+    subject = TextLine(
+        title=_("Subject"),
+        required=False,
+        )
+
     body = Text(
         title=_("Message body")
         )
@@ -137,6 +142,7 @@ class IInterventionGoal(IContained):
 
     presenting_concerns = Text(
         title=_("Presenting concerns"),
+        required=False,
         )
 
     goal = Text(
@@ -145,14 +151,17 @@ class IInterventionGoal(IContained):
 
     strengths = Text(
         title=_("Strengths"),
+        required=False,
         )
 
     indicators = Text(
         title=_("Indicators"),
+        required=False,
         )
 
     intervention = Text(
         title=_("Intervention"),
+        required=False,
         )
 
     timeline = Date(
@@ -161,6 +170,9 @@ class IInterventionGoal(IContained):
 
     persons_responsible = PersonListField(
         title=_("Persons responsible"),
+        description=_(u"The selected people will receive messages regarding"
+                        " this goal through their interventions inbox and"
+                        " email (if selected below)."),
         )
 
     at_one_time_responsible = List(
